/* This file is automatically generated.  DO NOT EDIT! */
extern ld_emulation_xfer_type ld_aarch64linux_emulation;
extern ld_emulation_xfer_type ld_aarch64elf_emulation;
extern ld_emulation_xfer_type ld_aarch64elf32_emulation;
extern ld_emulation_xfer_type ld_aarch64elf32b_emulation;
extern ld_emulation_xfer_type ld_aarch64elfb_emulation;
extern ld_emulation_xfer_type ld_armelf_emulation;
extern ld_emulation_xfer_type ld_armelfb_emulation;
extern ld_emulation_xfer_type ld_aarch64linuxb_emulation;
extern ld_emulation_xfer_type ld_aarch64linux32_emulation;
extern ld_emulation_xfer_type ld_aarch64linux32b_emulation;
extern ld_emulation_xfer_type ld_armelfb_linux_eabi_emulation;
extern ld_emulation_xfer_type ld_armelf_linux_eabi_emulation;

#define EMULATION_LIST \
  &ld_aarch64linux_emulation, \
  &ld_aarch64elf_emulation, \
  &ld_aarch64elf32_emulation, \
  &ld_aarch64elf32b_emulation, \
  &ld_aarch64elfb_emulation, \
  &ld_armelf_emulation, \
  &ld_armelfb_emulation, \
  &ld_aarch64linuxb_emulation, \
  &ld_aarch64linux32_emulation, \
  &ld_aarch64linux32b_emulation, \
  &ld_armelfb_linux_eabi_emulation, \
  &ld_armelf_linux_eabi_emulation, \
  0
